/*
 * Gnophone: A client for the Asterisk PBX
 *
 * Copyright (C) 2000, Linux Support Services, Inc.
 *
 * Written by Mark Spencer
 *
 * Linux/UNIX version distributed under the terms of
 * the GNU General Public License
 *
 * dialog.c Dialog windows for GnoPhone
 *
 *
 * This file was created by Rob Flynn. See me if you have a problem with it. OR ELSE! :-)
 */

#include <stdio.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <sys/signal.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <gdk/gdkkeysyms.h>
#include <gnophone.h>
#include <dialog.h>
#include <conversation.h>
#include <phonecore.h>


/* Load up some icons */
#include "../icons/cancel.xpm"
#include "../icons/mic.xpm"

/* Set up some dialog stuff */
static GList *dialogwindows = NULL;

/* Windows that we only want one instance of should be defined her */
/* static GtkWidget *examplewidget = NULL; */

/* Definition of our dialog windows goes here */

struct link_request_dialog {
	GtkWidget *window;
	char url[2048];
	int pcid;
};

static void destroy_dialog(GtkWidget *w, GtkWidget *w2)
{
	GtkWidget *dest;

	if (!GTK_IS_WIDGET(w2))
		dest = w;
	else
		dest = w2;

	dialogwindows = g_list_remove(dialogwindows, dest);
	gtk_widget_destroy(dest);
}

void deny_link_cb(GtkWidget *w, gpointer *data)
{
	struct link_request_dialog *l = (struct link_request_dialog *)data;
	pc_event e;

	e.event = PC_EVENT_LINKREJECT;
	e.len = sizeof(pc_event);
	e.callno = l->pcid;

	pc_write_event(SOURCE_GUI, &e);

	destroy_dialog(NULL, l->window);
}

void accept_link_cb(GtkWidget *w, gpointer *data)
{
	struct link_request_dialog *l = (struct link_request_dialog *)data;
	struct peerwindow * tmppeer = peers;
	struct peerwindow *p = NULL;
	pc_event e;

	/* First, we should unlink with anyone that we don't love :-) */
	if (link_pcid >= 0) {
		e.event = PC_EVENT_UNLINK;
		e.len = sizeof(pc_event);
		e.callno = l->pcid;

		pc_write_event(SOURCE_GUI, &e);

		link_pcid = -1;
	}

	while (tmppeer) {
		/* Uncheck everyone */
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmppeer->linked), FALSE);
		
		tmppeer = tmppeer->next;
	}

	/* Now check the winner =) */
	p = find_peerwindow(l->pcid);
	if (p) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p->linked), TRUE);
		link_pcid = l->pcid;
	}

	interpret_call(l->url);
	
	destroy_dialog(NULL, l->window);
}

void show_link_request_dialog(int pcid, char *name, char *url)
{
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *mbox;
	GtkWidget *acceptbtn;
	GtkWidget *denybtn;
	char buf[2048];

	struct link_request_dialog *w = g_new0(struct link_request_dialog, 1);

	g_snprintf(w->url, sizeof(w->url), "%s", url); /* copy this thing down */

	w->window = gtk_window_new(GTK_WINDOW_DIALOG);
	dialogwindows = g_list_prepend(dialogwindows, w->window);
	gtk_signal_connect(GTK_OBJECT(w->window), "delete_event", GTK_SIGNAL_FUNC(destroy_dialog), w->window);
	gtk_window_set_title(GTK_WINDOW(w->window), "Gnophone: Browser Link Request");
	gtk_widget_realize(w->window);
	
	/* Make our boxes */	
	vbox = gtk_vbox_new(FALSE, 0);
	hbox = gtk_hbox_new(FALSE, 5);

	g_snprintf(buf, sizeof(buf), "Browser Link Request from %s", name);
	frame = gtk_frame_new(buf);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 5);

	mbox = gtk_vbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(mbox), 5);

	gtk_container_add(GTK_CONTAINER(frame), mbox);

	/* Make a label */
	g_snprintf(buf, sizeof(buf), "You have received a browser link request from user '%s'. Your browser will be linked\nwith this user\'s upon accepting this request.  Any website the user views in the GnoPhone browser\nwill also be displayed on your browser and vice versa.\n\nAre you sure that you want to accept this link request?", name);
	label = gtk_label_new(buf);
	
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment(GTK_MISC(label), 0, .5);
	
	/* Make our buttons */
	acceptbtn = image_button_new(w->window, IMAGE_BUTTON_TYPE_BUTTON, "Accept", mic_xpm);
	gtk_widget_set_usize(acceptbtn, 90, 40);
	gtk_signal_connect(GTK_OBJECT(acceptbtn), "clicked", GTK_SIGNAL_FUNC(accept_link_cb), w);

	denybtn = image_button_new(w->window, IMAGE_BUTTON_TYPE_BUTTON, "Deny", cancel_xpm);
	gtk_widget_set_usize(denybtn, 90, 40);
	gtk_signal_connect(GTK_OBJECT(denybtn), "clicked", GTK_SIGNAL_FUNC(deny_link_cb), w);

	/* Pack them into the boxes */
	gtk_box_pack_end(GTK_BOX(hbox), denybtn, FALSE, FALSE, 5);
	gtk_box_pack_end(GTK_BOX(hbox), acceptbtn, FALSE, FALSE, 5);

	gtk_box_pack_start(GTK_BOX(mbox), label, FALSE, FALSE, 5);

	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
	
	gtk_container_add(GTK_CONTAINER(w->window), vbox);
	gtk_widget_show_all(w->window);
}

